/* discover.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <pwd.h>
#include "main.h"
#include "task.h"
#include "session.h"
#include "scheduler.h"
#include "gui.h"
#include "discover.h"
#include "filtermodule.h"
#include "licon.xpm"

/***************************************************************************
 Callbacks
 ***************************************************************************/

static void close_cb(GtkWidget *w, gpointer data)
{
	// stop discovery
	((Discover*)data)->hide();
}

static void next_cb(GtkWidget *w, gpointer data)
{ ((Discover*)data)->setpage(1); }
static void back_cb(GtkWidget *w, gpointer data)
{ ((Discover*)data)->setpage(-1); }
static void use_filter_cb(GtkWidget *w, gpointer data)
{
//	Discover *d = (Discover*)data;
//	d->use_ip_filter=((d->use_ip_filter=='0')?'1':'0');
//	d->update_discover();
}
static void use_hops_cb(GtkWidget *w, gpointer data)
{
//	Discover *d = (Discover*)data;
//	d->use_hops=((d->use_hops=='0')?'1':'0');
//	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->checkbutton18), ((d->use_hops=='1')?TRUE:FALSE));
}
static void change_num_hops_cb(GtkWidget *w, gpointer data)
{
//	Discover *d = (Discover*)data;
//	d->num_hops = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(d->spinbutton8));
//	d->update_discover();
}

/***************************************************************************
 Discover Class
 ***************************************************************************/

static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	((Discover*)data)->hide();
	return TRUE;
}

Discover::Discover(Main *m)
{
  GtkWidget *vbox69;
  GtkWidget *vbox70;
  GtkWidget *label189;
  GtkWidget *vbox72;
  GtkWidget *hbox96;
  GtkWidget *label196;
  GtkWidget *checkbutton16;
  GtkWidget *vbox73;
  GtkWidget *hbox97;
  GtkWidget *label197;
  GtkWidget *checkbutton19;
  GtkWidget *hbox95;
  GtkWidget *label193;
  GtkWidget *label194;
  GSList *vbox73_group = NULL;
  GtkWidget *radiobutton17;
  GtkWidget *radiobutton18;
  GtkWidget *radiobutton19;
  GtkWidget *table14;
  GtkWidget *label195;
  GSList *table14_group = NULL;
  GtkWidget *radiobutton20;
  GtkWidget *entry44;
  GtkWidget *radiobutton21;
  GtkWidget *vbox74;
  GtkWidget *scrolledwindow36;
  GtkWidget *label199;
  GtkWidget *hseparator5;
  GtkWidget *hbox93;
  GtkWidget *button101;

	main = m;
	page = 0;

	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (window), "Device Discovery");
	gtk_widget_realize(window);
	window_icon(window, licon_xpm);
	gdk_window_set_icon_name(window->window, "Device Discovery");
	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
		GTK_SIGNAL_FUNC(delete_event), this);

  vbox69 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox69);
  gtk_container_add (GTK_CONTAINER (window), vbox69);
  gtk_container_set_border_width (GTK_CONTAINER (vbox69), 4);

  notebook6 = gtk_notebook_new ();
  gtk_widget_show (notebook6);
  gtk_box_pack_start (GTK_BOX (vbox69), notebook6, TRUE, TRUE, 0);
  GTK_WIDGET_UNSET_FLAGS (notebook6, GTK_CAN_FOCUS);
  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook6), FALSE);
  gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook6), FALSE);

  vbox70 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox70);
  gtk_container_add (GTK_CONTAINER (notebook6), vbox70);

  label189 = gtk_label_new ("Discovery is restricted to those network devices that have\nan SNMP community string in the following list");
  gtk_widget_show (label189);
  gtk_box_pack_start (GTK_BOX (vbox70), label189, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label189), GTK_JUSTIFY_FILL);
  gtk_misc_set_alignment (GTK_MISC (label189), 0, 0.5);

  vbox72 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox72);
  gtk_container_add (GTK_CONTAINER (notebook6), vbox72);

  hbox96 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox96);
  gtk_box_pack_start (GTK_BOX (vbox72), hbox96, FALSE, FALSE, 0);

  label196 = gtk_label_new ("Restrict discovery to devices that match the following IP filter");
  gtk_widget_show (label196);
  gtk_box_pack_start (GTK_BOX (hbox96), label196, FALSE, FALSE, 0);

  checkbutton16 = gtk_check_button_new_with_label ("");
  gtk_widget_show (checkbutton16);
  gtk_box_pack_start (GTK_BOX (hbox96), checkbutton16, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (checkbutton16), "clicked",
		GTK_SIGNAL_FUNC (use_filter_cb), this);

	filtermodule = new FilterModule(window);
	gtk_box_pack_start (GTK_BOX (vbox72), filtermodule->vbox58, TRUE, TRUE, 0);

  vbox73 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox73);
  gtk_container_add (GTK_CONTAINER (notebook6), vbox73);

  hbox97 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox97);
  gtk_box_pack_start (GTK_BOX (vbox73), hbox97, TRUE, TRUE, 0);

  label197 = gtk_label_new ("Rediscover devices already found");
  gtk_widget_show (label197);
  gtk_box_pack_start (GTK_BOX (hbox97), label197, FALSE, FALSE, 0);

  checkbutton19 = gtk_check_button_new_with_label ("");
  gtk_widget_show (checkbutton19);
  gtk_box_pack_start (GTK_BOX (hbox97), checkbutton19, FALSE, FALSE, 0);

  hbox95 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox95);
  gtk_box_pack_start (GTK_BOX (vbox73), hbox95, FALSE, TRUE, 0);

  label193 = gtk_label_new ("Restrict discovery to");
  gtk_widget_show (label193);
  gtk_box_pack_start (GTK_BOX (hbox95), label193, FALSE, FALSE, 0);

  spinbutton8_adj = gtk_adjustment_new (1, 0, 64, 1, 1, 1);
  spinbutton8 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton8_adj), 1, 0);
  gtk_widget_show (spinbutton8);
  gtk_box_pack_start (GTK_BOX (hbox95), spinbutton8, FALSE, TRUE, 0);
	gtk_signal_connect (GTK_OBJECT(spinbutton8_adj), "value_changed",
		GTK_SIGNAL_FUNC (change_num_hops_cb), this);

  label194 = gtk_label_new ("hops");
  gtk_widget_show (label194);
  gtk_box_pack_start (GTK_BOX (hbox95), label194, FALSE, FALSE, 0);

  checkbutton18 = gtk_check_button_new_with_label ("");
  gtk_widget_show (checkbutton18);
  gtk_box_pack_start (GTK_BOX (hbox95), checkbutton18, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (checkbutton18), "clicked",
		GTK_SIGNAL_FUNC (use_hops_cb), this);

  radiobutton17 = gtk_radio_button_new_with_label (vbox73_group, "Discover routers");
  vbox73_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton17));
  gtk_widget_show (radiobutton17);
  gtk_box_pack_start (GTK_BOX (vbox73), radiobutton17, FALSE, FALSE, 0);

  radiobutton18 = gtk_radio_button_new_with_label (vbox73_group, "Discover routers & switches");
  vbox73_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton18));
  gtk_widget_show (radiobutton18);
  gtk_box_pack_start (GTK_BOX (vbox73), radiobutton18, FALSE, FALSE, 0);

  radiobutton19 = gtk_radio_button_new_with_label (vbox73_group, "Discover everything matching Filter Rules");
  vbox73_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton19));
  gtk_widget_show (radiobutton19);
  gtk_box_pack_start (GTK_BOX (vbox73), radiobutton19, FALSE, FALSE, 0);

  table14 = gtk_table_new (2, 3, FALSE);
  gtk_widget_show (table14);
  gtk_box_pack_start (GTK_BOX (vbox73), table14, TRUE, TRUE, 0);
  gtk_table_set_row_spacings (GTK_TABLE (table14), 2);
  gtk_table_set_col_spacings (GTK_TABLE (table14), 4);

  label195 = gtk_label_new ("Start with");
  gtk_widget_show (label195);
  gtk_table_attach (GTK_TABLE (table14), label195, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label195), 0, 0.5);

  radiobutton20 = gtk_radio_button_new_with_label (table14_group, "Router");
  table14_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton20));
  gtk_widget_show (radiobutton20);
  gtk_table_attach (GTK_TABLE (table14), radiobutton20, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  entry44 = gtk_entry_new ();
  gtk_widget_show (entry44);
  gtk_table_attach (GTK_TABLE (table14), entry44, 2, 3, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  radiobutton21 = gtk_radio_button_new (table14_group);
  table14_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton21));
  gtk_widget_show (radiobutton21);
  gtk_table_attach (GTK_TABLE (table14), radiobutton21, 1, 3, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  label200 = gtk_label_new ("Default Gateway");
  gtk_widget_show (label200);
  gtk_container_add (GTK_CONTAINER (radiobutton21), label200);
  gtk_misc_set_alignment (GTK_MISC (label200), 0, 0.5);

  vbox74 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox74);
  gtk_container_add (GTK_CONTAINER (notebook6), vbox74);

  label198 = gtk_label_new ("Current task message here");
  gtk_widget_show (label198);
  gtk_box_pack_start (GTK_BOX (vbox74), label198, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label198), 7.45058e-09, 0.5);

  scrolledwindow36 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow36);
  gtk_box_pack_start (GTK_BOX (vbox74), scrolledwindow36, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow36), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  clist33 = gtk_clist_new (2);
  gtk_widget_show (clist33);
  gtk_container_add (GTK_CONTAINER (scrolledwindow36), clist33);
  gtk_clist_set_column_width (GTK_CLIST (clist33), 0, 16);
  gtk_clist_set_column_width (GTK_CLIST (clist33), 1, 64);
  gtk_clist_column_titles_hide (GTK_CLIST (clist33));

  label199 = gtk_label_new ("label199");
  gtk_widget_show (label199);
  gtk_clist_set_column_widget (GTK_CLIST (clist33), 0, label199);

  hseparator5 = gtk_hseparator_new ();
  gtk_widget_show (hseparator5);
  gtk_box_pack_start (GTK_BOX (vbox69), hseparator5, FALSE, TRUE, 0);

  hbox93 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox93);
  gtk_box_pack_start (GTK_BOX (vbox69), hbox93, FALSE, TRUE, 0);

  button101 = gtk_button_new_with_label ("Cancel");
  gtk_widget_show (button101);
  gtk_box_pack_end (GTK_BOX (hbox93), button101, FALSE, FALSE, 0);
  gtk_widget_set_usize (button101, 60, -2);
  gtk_signal_connect (GTK_OBJECT (button101), "clicked",
		GTK_SIGNAL_FUNC (close_cb), this);

  button100 = gtk_button_new_with_label ("Next");
  gtk_widget_show (button100);
  gtk_box_pack_end (GTK_BOX (hbox93), button100, FALSE, FALSE, 0);
  gtk_widget_set_usize (button100, 60, -2);
  gtk_signal_connect (GTK_OBJECT (button100), "clicked",
		GTK_SIGNAL_FUNC (next_cb), this);

  button99 = gtk_button_new_with_label ("Back");
  gtk_widget_show (button99);
  gtk_box_pack_end (GTK_BOX (hbox93), button99, FALSE, FALSE, 0);
  gtk_widget_set_usize (button99, 60, -2);
  gtk_signal_connect (GTK_OBJECT (button99), "clicked",
		GTK_SIGNAL_FUNC (back_cb), this);

	// set defaults
	use_ip_filter = '0';
	ip_filter.empty();
	rediscover = '1';
	use_hops = '1';
	num_hops = 3;
	discover_level = 3;
	get_gateway();
	start_address.set(gateway);
}

Discover::~Discover()
{
	if (filtermodule!=NULL) delete filtermodule;
}

void Discover::show()
{
	char buf[256];

	gtk_widget_show(window);
	gdk_window_raise(window->window);

	setpage(0);
	sprintf(buf, "Default Gateway (%s)", gateway.getstring());
	gtk_label_set_text(GTK_LABEL(label200), buf);
	update_discover();
}

void Discover::hide()
{
	gtk_widget_hide(window);
}

void Discover::setpage(int step)
{
//	char buf[256];

	if (step==0) page = 0; else page += step;

	gtk_widget_set_sensitive(button99, TRUE);
	gtk_widget_set_sensitive(button100, TRUE);
	gtk_label_set_text(GTK_LABEL(GTK_BIN (button100)->child), "Next");

	if (page==0) gtk_widget_set_sensitive(button99, FALSE);
	if (page==3) gtk_widget_set_sensitive(button100, FALSE);
	if (page==2)
		gtk_label_set_text(GTK_LABEL(GTK_BIN (button100)->child), "Start");

	if (page==3)
	{
//		sprintf(buf, "xxx.xxx.xxx.xxx %s",
//			main->session->options->getCommunityList());
//		main->scheduler->addTask(buf, 1, 2, 3);
	}

	gtk_notebook_set_page(GTK_NOTEBOOK (notebook6), page);
}

void Discover::update_discover()
{
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton18), ((use_hops=='1')?TRUE:FALSE));
	gtk_widget_set_sensitive(spinbutton8, ((use_hops=='1')?TRUE:FALSE));
//	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton8), num_hops);

//	GTK_ADJUSTMENT(spinbutton8_adj)->value = num_hops;
//	gtk_signal_emit_by_name (GTK_OBJECT (spinbutton8_adj), "changed");
//	gtk_widget_set_sensitive(filtermodule->vbox58, ((use_ip_filter=='1')?TRUE:FALSE));
}

void Discover::get_gateway()
{
	FILE *f;
	char buff[1024], junk[1024];
	char addr1[16], addr2[16], addr3[16];
	int val;

	f = fopen("/proc/net/route", "r");

	gateway.empty();
	if (f)
	{
		fgets(buff, 1023, f);
		while (fgets(buff, 1023, f))
		{
			sscanf(buff, "%[^\t]\t%[^\t]\t%[^\t]\t%X\t%[^\n]\n",
				addr1, addr2, addr3, &val, junk);
			if ((val & 2)==2)
			{
				gateway.setafinet(addr3);
			}
		}
		fclose(f);
	}
}

void Discover::update_task_list()
{
	Link *ln;
	Task *t;
	gchar *data[2];
	char buf[64];
	int lvl;

	gtk_clist_freeze(GTK_CLIST(clist33));
	gtk_clist_clear(GTK_CLIST(clist33));
	for (lvl=1; lvl<=6; lvl++)
	{
		for (ln=main->scheduler->tasks.get_head(); ln!=NULL; ln=ln->next)
		{
			t = (Task*) ln->item;
			if ((t->type==1) && (t->priority==lvl))
			{
				sprintf(buf, "%d", lvl);
				data[0] = strdup(buf);
				data[1] = strdup(t->cmnd);
				gtk_clist_append(GTK_CLIST(clist33), data);
				delete data[0]; delete data[1];
			}
		}
	}
	gtk_clist_thaw(GTK_CLIST(clist33));
}

void Discover::discover_message(char *str)
{
	gtk_label_set_text(GTK_LABEL(label198), str);
}

