/* image.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <stdlib.h>
#include "image.h"

Image::Image()
{
	gc = NULL;
	pixmap = NULL;
	mask = NULL;
	pixbuf = NULL;
}

Image::~Image()
{
	if (gc!=NULL) gdk_gc_unref(gc);
	if (pixmap!=NULL) gdk_pixmap_unref(pixmap);
	if (mask!=NULL) gdk_bitmap_unref(mask);
	if (pixbuf!=NULL) gdk_pixbuf_unref(pixbuf);
}

void Image::load(GtkWidget *w, const char **data)
{
	if (gc!=NULL) gdk_gc_unref(gc);
	gc = gdk_gc_new(w->window);
	if (pixbuf) gdk_pixbuf_unref(pixbuf);
	pixbuf = gdk_pixbuf_new_from_xpm_data(data);
}

void Image::scale(int ww, int hh)
{
	GdkPixbuf *buf;

	w = ww;
	h = hh;
	buf = gdk_pixbuf_scale_simple(pixbuf, w, h, GDK_INTERP_NEAREST);
	if (pixmap) gdk_pixmap_unref(pixmap);
	if (mask) gdk_bitmap_unref(mask);
	gdk_pixbuf_render_pixmap_and_mask(buf, &pixmap, &mask, 255);
	if (buf) gdk_pixbuf_unref(buf);
	gdk_gc_set_clip_mask(gc, mask);
}

void Image::draw(GdkDrawable *drawable, int x, int y)
{
	gdk_gc_set_clip_origin(gc, x, y);
	gdk_draw_pixmap(drawable, gc, pixmap, 0, 0, x, y, w, h);
}

