/* data.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gnome-xml/tree.h>
#include <gnome-xml/parser.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "data.h"
#include "interface.h"
#include "network.h"
#include "machine.h"
#include "subnet.h"
#include "device.h"

Data::Data()
{
	width = 0;
	height = 0;
	minx = miny = 0;
	show_leaves = '1';
	empty();
}

Data::~Data()
{
	empty();
}

void Data::empty()
{
	Network *n;
	Machine *m;
	Subnet *s;
	Device *d;
	Link *ln;

	for (ln=subnets.get_head(); ln!=NULL; ln=ln->next)
	{ s = (Subnet*) ln->item; if (s!=NULL) delete s; }
	subnets.empty();
	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
	{ m = (Machine*) ln->item; if (m!=NULL) delete m; }
	machines.empty();
	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{ d = (Device*) ln->item; if (d!=NULL) delete d; }
	devices.empty();
	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{ n = (Network*) ln->item; if (n!=NULL) delete n; }
	networks.empty();
	strcpy(default_view, "");
}

void Data::savexml(char *fname)
{
	xmlDocPtr doc;
	Subnet *s;
	Device *d;
	Network *n;
	Link *ln;

	doc = xmlNewDoc((xmlChar*)"1.0");
	doc->root = xmlNewDocNode(doc, NULL, (xmlChar*)"Nomad_Topology", NULL);
	if (strcmp(default_view, "")!=0)
		xmlSetProp(doc->root, (xmlChar*)"default_view", (xmlChar*)default_view);

	for (ln=subnets.get_head(); ln!=NULL; ln=ln->next)
	{
		s = (Subnet*) ln->item;
		s->savexml(doc->root);
	}
	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		d->savexml(doc->root);
	}

	// save only those networks that are attached to stuff
	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
//		if (n->interfaces.size() + n->machines.size() > 1)
			n->savexml(doc->root);
	}

	xmlSaveFile(fname, doc);
	xmlFreeDoc(doc);
}

void Data::loadxml(char *fname)
{
	xmlChar *buf;
	xmlDocPtr doc;
	xmlNodePtr xmlnode;
	Device *d;
	Subnet *s;
	Network *n;
	doc = xmlParseFile(fname);

	if (!doc || !doc->root || !doc->root->name ||
			strcmp((char*)doc->root->name, "Nomad_Topology")!=0)
	{
		xmlFreeDoc(doc);
		return;
	}

	empty();

	buf = (xmlChar*)xmlGetProp(doc->root, (xmlChar*)"default_view");
	if (buf!=NULL) { strcpy(default_view, (char*)buf); free(buf); }
//	printf("DEFAULT VIEW = %s\n", default_view);

	xmlnode = doc->root->childs;
	while (xmlnode != NULL)
	{
		if (xmlnode->name!=NULL)
		{
			if (strcmp((char*)xmlnode->name, "Subnet")==0)
			{
				s = new Subnet();
				s->loadxml(xmlnode);
				subnets.tag(s);
				while(gtk_events_pending()) gtk_main_iteration();
			}else
			if (strcmp((char*)xmlnode->name, "Device")==0)
			{
				d = new Device();
				d->loadxml(xmlnode);
				devices.tag(d);
				while(gtk_events_pending()) gtk_main_iteration();
			}else
			if (strcmp((char*)xmlnode->name, "Network")==0)
			{
				n = new Network();
				n->loadxml(xmlnode, &devices, &machines, &subnets);
//				printf("LOAD network %d subnets\n", n->subnets.size());
				networks.tag(n);
				while(gtk_events_pending()) gtk_main_iteration();
			}
		}
		xmlnode = xmlnode->next;
	}

	xmlFreeDoc(doc);
}

void Data::print()
{
/*
	Link *ln;
	Device *d;
	Network *n;

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		printf("DEVICE   %f %f\n", d->rx, d->ry);
	}

	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		printf("NETWORK  %f %f\n", n->rx, n->ry);
	}
*/
}

// search all devices looking for an ip address
Device *Data::find_ip_on_device(IPAddress tip)
{
	IPAddress *ip;
	Link *ln, *ln2, *ln3;
	Device *d;
	Interface *in;

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		if (d->pingname.compare(tip)==0) return d;
		for (ln2=d->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			in = (Interface*) ln2->item;
			for (ln3=in->ip_addresses.get_head(); ln3!=NULL; ln3=ln3->next)
			{
				ip = (IPAddress*) ln3->item;
				if (ip->compare(tip)==0) return d;
			}
		}
	}
	return NULL;
}

void Data::select(char val)
{
	Link *ln;
	Device *r;
	Network *n;
	Machine *m;

	for (ln = devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		r->selected = '0';
		if (r->show=='1') r->selected = val;
	}

	for (ln = machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		m->selected = '0';
		if (m->show=='1') m->selected = val;
	}

	for (ln = networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		n->selected = '0';
		if (n->show=='1') n->selected = val;
	}
}

/*
void Data::hide_leaf_networks()
{
	Link *ln;
	Network *n;

	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->interfaces.size() < 2)
			n->show = 0;
	}
}
*/

void Data::size()
{
	Link *ln;
	Device *d;
	Network *n;
	Machine *m;
	long maxx, maxy;
	int match;

	if (devices.size()==0) return;

	ln=devices.get_head();
	d = (Device*) ln->item;
	minx = miny = 100000;
	maxx = maxy = -100000;
	match = 0;

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		if (minx > d->x-(d->w/2)) minx = d->x-(d->w/2);
		if (maxx < d->x+(d->w/2)) maxx = d->x+(d->w/2);
		if (miny > d->y-(d->h/2)) miny = d->y-(d->h/2);
		if (maxy < d->y+(d->h/2)) maxy = d->y+(d->h/2);
		if (d->show=='1') match = 1;
	}

	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		if (minx > m->x-(m->w/2)) minx = m->x-(m->w/2);
		if (maxx < m->x+(m->w/2)) maxx = m->x+(m->w/2);
		if (miny > m->y-(m->h/2)) miny = m->y-(m->h/2);
		if (maxy < m->y+(m->h/2)) maxy = m->y+(m->h/2);
		if (m->show=='1') match = 1;
	}

	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (minx > n->x-(n->w/2)) minx = n->x-(n->w/2);
		if (maxx < n->x+(n->w/2)) maxx = n->x+(n->w/2);
		if (miny > n->y-(n->h/2)) miny = n->y-(n->h/2);
		if (maxy < n->y+(n->h/2)) maxy = n->y+(n->h/2);
		if (n->show=='1') match = 1;
	}

	if (match==0)
	{
		minx = miny = 0;
		return;
	}

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		d->x-=minx; d->y-=miny;
	}
	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		m->x-=minx; m->y-=miny;
	}
	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		n->x-=minx; n->y-=miny;
	}

	if (match==1)
	{
		width = (maxx-minx);
		height = (maxy-miny);
	}
}

void Data::sizeshow()
{
	Link *ln;
	Device *d;
	Network *n;
	Machine *m;
	long maxx, maxy;
	int match;

	if (devices.size()==0) return;

	ln=devices.get_head();
	d = (Device*) ln->item;
	minx = miny = 100000;
	maxx = maxy = -100000;
	match = 0;

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		if (d->show=='1')
		{
			if (minx > d->x-(d->w/2)) minx = d->x-(d->w/2);
			if (maxx < d->x+(d->w/2)) maxx = d->x+(d->w/2);
			if (miny > d->y-(d->h/2)) miny = d->y-(d->h/2);
			if (maxy < d->y+(d->h/2)) maxy = d->y+(d->h/2);
			match = 1;
		}
	}

	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		if (m->show=='1')
		{
			if (minx > m->x-(m->w/2)) minx = m->x-(m->w/2);
			if (maxx < m->x+(m->w/2)) maxx = m->x+(m->w/2);
			if (miny > m->y-(m->h/2)) miny = m->y-(m->h/2);
			if (maxy < m->y+(m->h/2)) maxy = m->y+(m->h/2);
			match = 1;
		}
	}

	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->show=='1')
		{
			if (minx > n->x-(n->w/2)) minx = n->x-(n->w/2);
			if (maxx < n->x+(n->w/2)) maxx = n->x+(n->w/2);
			if (miny > n->y-(n->h/2)) miny = n->y-(n->h/2);
			if (maxy < n->y+(n->h/2)) maxy = n->y+(n->h/2);
			match = 1;
		}
	}

	if (match==0)
	{
		minx = miny = 0;
		return;
	}

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		d->x-=minx; d->y-=miny;
	}
	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		d->x-=minx; d->y-=miny;
	}
	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		n->x-=minx; n->y-=miny;
	}

	if (match==1)
	{
		width = (maxx-minx);
		height = (maxy-miny);
	}
}

int Data::selected_size()
{
	Link *ln;
	Device *d;
	Network *n;
	Machine *m;
	long mnx, mny, mxx, mxy;
	int cnt = 0;

	if (devices.size()==0) return 0;

	ln=devices.get_head();
	d = (Device*) ln->item;
	mnx = mny = 100000;
	mxx = mxy = -100000;

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		if ((d->show=='1') && (d->selected=='1'))
		{
			if (mnx > d->x-(d->w/2)) mnx = d->x-(d->w/2);
			if (mxx < d->x+(d->w/2)) mxx = d->x+(d->w/2);
			if (mny > d->y-(d->h/2)) mny = d->y-(d->h/2);
			if (mxy < d->y+(d->h/2)) mxy = d->y+(d->h/2);
			cnt = 1;
		}
	}

	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		if ((m->show=='1') && (m->selected=='1'))
		{
			if (mnx > m->x-(m->w/2)) mnx = m->x-(m->w/2);
			if (mxx < m->x+(m->w/2)) mxx = m->x+(m->w/2);
			if (mny > m->y-(m->h/2)) mny = m->y-(m->h/2);
			if (mxy < m->y+(m->h/2)) mxy = m->y+(m->h/2);
			cnt = 1;
		}
	}

	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if ((n->show=='1') && (n->selected=='1'))
		{
			if (mnx > n->x-(n->w/2)) mnx = n->x-(n->w/2);
			if (mxx < n->x+(n->w/2)) mxx = n->x+(n->w/2);
			if (mny > n->y-(n->h/2)) mny = n->y-(n->h/2);
			if (mxy < n->y+(n->h/2)) mxy = n->y+(n->h/2);
			cnt = 1;
		}
	}

	width = (mxx-mnx);
	height = (mxy-mny);
	minx = mnx;
	miny = mny;

	return cnt;
}

void Data::set_leaves()
{
	Link *ln;
	Network *n;

	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if ((n->interfaces.size() < 2) && (show_leaves=='0'))
			n->show = '0';
		else
			n->show = '1';
	}
}

void Data::hide()
{
	Device *r;
	Network *n;
	Machine *m;
	Link *ln;

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		if (r->selected == '1') r->show = '0';
	}
	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		if (m->selected == '1') m->show = '0';
	}
	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->selected == '1') n->show = '0';
	}
}

void Data::buncha(Device *r, int sel)
{
	Link *ln, *ln2;
	double radius, angle;
	int l, total, ring, machine_count;
	Interface *in;
	Machine *m;

	if (r->show=='0') return;
	if (sel) r->selected = '1';

	l = 0;
	for (ln2 = r->interfaces.get_head(); ln2!=NULL; ln2 = ln2->next)
	{
		in = (Interface*) ln2->item;
		if (in->network != NULL)
		{
			total = ring = 0;
			while (total<l+1) { ring++; total += (8*ring); }

			radius = (((80*ring)+40) * 8)/(2*3.14159);
			angle = 2*3.14159/(ring*8);

			if ((in->network->show=='1') && (is_network_a_leaf(in->network)))
			{
				in->network->x = r->x + int(radius * cos(angle*(total-l)));
				in->network->y = r->y + int(radius * sin(angle*(total-l)));
				if (sel) in->network->selected = '1';
				l++;
			}
		}
	}
}

void Data::bunch(Visual *cur, int sel)
{
	Link *ln;
	Device *r;

	if ((cur != NULL) && ((cur->type == 'r') || (cur->type == 's')) &&
			(cur->selected == '0'))
	{
		select('0');
		buncha((Device*)cur, sel);
	}
	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		if (r->selected == '1') buncha(r, sel);
	}
}

void Data::group_all(char ch)
{
	Link *ln;
	Device *r;

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		r->group = ch;
	}
}

int Data::is_network_a_leaf(Network *n)
{
	int machine_count;
	Machine *m;
	Link *ln;

	machine_count=0;
	for (ln = n->machines.get_head(); ln!=NULL; ln = ln->next)
	{
		m = (Machine*) ln->item;
		if (m->show=='1') machine_count++;
	}

	return ((n->interfaces.size() == 1) && (machine_count == 0));
}

void Data::stara(Device *r, int sel)
{
	Link *ln, *ln2;
	double radius, angle;
	int l, machine_count;
	Interface *in;
	Machine *m;

	if (r->show=='0') return;
	if (sel) r->selected = '1';

	l = 0;
	for (ln2 = r->interfaces.get_head(); ln2!=NULL; ln2 = ln2->next)
	{
		in = (Interface*) ln2->item;
		if ((in->network!=NULL) && (in->network->show=='1') &&
				(is_network_a_leaf(in->network))) l++;
	}

	radius = ((((l<10)?10:l)*96)+64)/(2*3.14159);
	angle = (2*3.14159)/l;

	l = 0;
	for (ln2 = r->interfaces.get_head(); ln2!=NULL; ln2 = ln2->next)
	{
		in = (Interface*) ln2->item;
		if ((in->network!=NULL) && (in->network->show=='1') &&
				(is_network_a_leaf(in->network)))
		{
			in->network->x = r->x + int(radius * cos(angle*l));
			in->network->y = r->y - int(radius * sin(angle*l));
			if (sel) in->network->selected = '1';
			l++;
		}
	}
}

void Data::star(Visual *cur, int sel)
{
	Link *ln;
	Device *r;

	if ((cur != NULL) && ((cur->type == 'r') || (cur->type == 's')) &&
			(cur->selected == '0'))
	{
		select('0');
		stara((Device*)cur, sel);
	}
	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		if (r->selected == '1') stara(r, sel);
	}
}

void Data::group_machines()
{
	Link *ln, *ln2;
	Network *n;
	Machine *m;
	int count, val, xlen;
	long tx, ty;

	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->group=='b')
		{
			count = 0;
			for (ln2=n->machines.get_head(); ln2!=NULL; ln2=ln2->next)
			{
				m = (Machine*) ln2->item;
				if (m->show=='1') count++;
			}
			if (count>0)
			{
				val = 0;
				xlen = (int) sqrt(count)+1;
				printf("count=%d, xlen=%d\n", count, xlen);
				for (ln2=n->machines.get_head(); ln2!=NULL; ln2=ln2->next)
				{
					m = (Machine*) ln2->item;
					if (m->show=='1')
					{
						m->x = n->x + (2 * m->w * (1+(val%xlen)));
						m->y = n->y + (2 * m->h * (1+(val/xlen)));
						val++;
					}
				}
			}
		}
	}
}

void Data::apply_group()
{
	Link *ln, *ln2;
	Interface *in;
	Device *r;
	Network *n;
	long tx, ty;

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		if (r->group=='b') buncha(r, 0); else
		if (r->group=='s') stara(r, 0);
	}

	// extra code to automove p-to-p networks
	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if ((n->view=='p') && (n->interfaces.size() > 1))
		{
			tx = ty = 0;
			for (ln2=n->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
			{
				in = (Interface*) ln2->item;
				r = in->device;
				tx += r->x;
				ty += r->y;
			}
			n->x = tx/n->interfaces.size();
			n->y = ty/n->interfaces.size();
		}
	}
}

void Data::show_all()
{
	Link *ln;
	Device *r;
	Network *n;

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		r->show='1';
	}

	// showing all machines would be a bad idea

	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		n->show='1';
	}
}

void Data::del_device(Device *d)
{
	Link *ln;
	Interface *in;

	if (d==NULL) return;

	for (ln=d->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		if ((in!=NULL)&&(in->network!=NULL))
		{
			in->network->interfaces.del(in);
			if (in->network->interfaces.size()==0)
			{
				networks.del(in->network);
				delete (in->network);
			}
		}
	}
	devices.del(d);
	delete d;
}

/*
void Data::autoplace_init()
{
	Link *ln, *ln2, *ln3;
	Edge *e;
	Node *nd;
	Network *n;
	Interface *in;
	Node *n1;
	Visual *v, *v2;
	int countr, counts;
	Device *r;

// delete the old lists here
	for (ln=nodes.get_head(); ln!=NULL; ln=ln->next)
	{
		nd = (Node*) ln->item;
		if (nd!=NULL) delete nd;
	}
	nodes.empty();
	for (ln=edges.get_head(); ln!=NULL; ln=ln->next)
	{
		e = (Edge*) ln->item;
		if (e!=NULL) delete e;
	}
	edges.empty();

// start by building a list of nodes

	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		v = (Visual*) ln->item;
		v->fix = 0;
		if (v->show == '1')
		{
			nd = new Node();
			nd->dx = 0;
			nd->dy = 0;
			nd->obj = v;
			nodes.add(nd);
		}
	}
	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		v = (Visual*) ln->item;
		v->fix = 0;
		if (v->show == '1')
		{
			nd = new Node();
			nd->dx = 0;
			nd->dy = 0;
			nd->obj = v;
			nodes.add(nd);
		}
	}

// now build a list of edges

	for (ln=nodes.get_head(); ln!=NULL; ln=ln->next)
	{
		nd = (Node*) ln->item;
		v = (Visual*) nd->obj;
		if (v->type == 'n')
		{
			n = (Network*) v;
			if (n->show=='1')		// for each network node
			{
				countr = 0;
				counts = 0;
				for (ln2=n->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
				{
					in = (Interface*) ln2->item;
					r = in->device;
					if (r->type == 'r') countr++;
					if (r->type == 's') counts++;
				}

// first count the number of networks attached to only one device
// set edges for these networks
// set count in devices if current count = 0
// set count in networks if current count = 0
// goto line 3 until all counts are set


//				if (countr+counts==1)
//				{
//				}


				// this one works
				for (ln2=n->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
				{
					in = (Interface*) ln2->item;
					r = in->device;
					for (ln3=nodes.get_head(); ln3!=NULL; ln3=ln3->next)
					{
						n1 = (Node*) ln3->item;
						v2 = (Visual*) n1->obj;
						if (v2 == r)
						{
							e = new Edge();
							e->from = n1;
							e->to = nd;
							e->len = 400+(countr*50)+(counts*25);
							edges.add(e);
						}
					}
				}
			}
		}
	}
}

Node *Data::get_node(Visual*v)
{
	Link *ln;
	Node *nd;

	for (ln=nodes.get_head(); ln!=NULL; ln=ln->next)
	{
		nd = (Node*) ln->item;
		if (nd->obj==v)
			return nd;
	}
	return NULL;
}

void Data::autoplace()
{
	Link *ln, *ln2;
	Edge *e;
	Node *n1, *n2;
	double vx, vy, dlen, len, f, dx, dy;

//	printf("AUTO 1\n");
	for (ln=edges.get_head(); ln!=NULL; ln=ln->next)
	{
		e = (Edge*) ln->item;
		vx = e->to->obj->x - e->from->obj->x;
		vy = e->to->obj->y - e->from->obj->y;
		len = sqrt(vx*vx + vy*vy);
		if (len<50) len=50;
		f = (e->len - len) / (len*3);
		dx = f * vx;
		dy = f * vy;

		e->to->dx += dx;
		e->to->dy += dy;
		e->from->dx += -dx;
		e->from->dy += -dy;
	}

//	printf("AUTO 2\n");
	for (ln=nodes.get_head(); ln!=NULL; ln=ln->next)
	{
		n1 = (Node*) ln->item;
		dx = 0;
		dy = 0;
		while(gtk_events_pending()) gtk_main_iteration();
		for (ln2=nodes.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			n2 = (Node*) ln2->item;
			if (n1!=n2)
			{
				vx = n1->obj->x - n2->obj->x;
				vy = n1->obj->y - n2->obj->y;
				len = vx*vx + vy*vy;
				if (len == 0)
				{
					dx += rand()%10;
					dy += rand()%10;
				}else
				if (len < 100*100)
				{
					dx += vx / len;
					dy += vy / len;
				}
			}
		}
		dlen = dx*dx + dy*dy;
		if (dlen>0)
		{
			dlen = sqrt(dlen) / 2;
			n1->dx += dx / dlen;
			n1->dy += dy / dlen;
		}
	}

//	printf("AUTO 3\n");
	for (ln=nodes.get_head(); ln!=NULL; ln=ln->next)
	{
		n1 = (Node*) ln->item;
		if (n1->obj->fix == 0)
		{
			if (n1->dx<5) vx=n1->dx; else vx=5;
			if (vx<-5) vx=-5;
			n1->obj->x += (int) vx;
			if (n1->dy<5) vy=n1->dy; else vy=5;
			if (vy<-5) vy=-5;
			n1->obj->y += (int) vy;
			n1->dx /= 1.5;
			n1->dy /= 1.5;
		}
	}
}
*/

//
// work out position of network relative to visible devices and machines
//

void Data::position_links()
{
	Link *ln, *ln2;
	Network *n;
	Device *r;
	Machine *m;
	Interface *in;
	long tx, ty;
	int count;

	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if ((n->position=='a') && (!is_network_a_leaf(n)))
		{
			count = 0;
			tx = ty = 0;
			for (ln2=n->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
			{
				in = (Interface*) ln2->item;
				r = in->device;
				if (r->show=='1')
				{
					tx += r->x;
					ty += r->y;
					count++;
				}
			}
			for (ln2=n->machines.get_head(); ln2!=NULL; ln2=ln2->next)
			{
				m = (Machine*) ln2->item;
				if (m->show=='1')
				{
					tx += m->x;
					ty += m->y;
					count++;
				}
			}
			if (count>0)
			{
				n->x = tx/count;
				n->y = ty/count;
			}
		}
	}
}

